// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all <standard< wxWindows headers
#ifndef WX_PRECOMP
    #include <wx/wx.h>

    #include <wx/log.h>
#endif

#include <wx/caret.h>
#include <wx/settings.h>

#include "dumpwin.h"

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// DumpWindow
// ----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(DumpWindow, wxWindow)

BEGIN_EVENT_TABLE(DumpWindow, wxWindow)
    EVT_PAINT(DumpWindow::OnPaint)
    EVT_SIZE(DumpWindow::OnSize)
    EVT_CHAR(DumpWindow::OnChar)
    EVT_SCROLLWIN(DumpWindow::OnScrollWin)
END_EVENT_TABLE()

DumpWindow::DumpWindow( wxWindow *parent )
        : wxWindow( parent, -1,
                            wxDefaultPosition, wxDefaultSize,
                            0 ), y_scroll(0), x_scroll(0)
{
//    m_text = (wxChar *)NULL;

    SetBackgroundColour(* wxWHITE);

//    m_font = wxSystemSettings::GetSystemFont(wxSYS_ANSI_FIXED_FONT);
    m_font = wxSystemSettings::GetSystemFont(wxSYS_ANSI_VAR_FONT);
//    m_font = wxFont(m_font.GetPointSize(),wxMODERN,m_font.GetStyle(),
//		    m_font.GetWeight(),m_font.GetUnderlined(),
//		    m_font.GetFaceName());

    wxClientDC dc(this);
    dc.SetFont( m_font );
    m_heightChar = dc.GetCharHeight();
    m_widthChar = dc.GetCharWidth();

//    wxCaret *caret = new wxCaret(this, m_widthChar, m_heightChar);
//    SetCaret(caret);

//    m_xCaret = m_yCaret =
    m_xChars = m_yChars = 0;

    m_xMargin = m_yMargin = 1;
//    caret->Move(m_xMargin, m_yMargin);
//    caret->Show();

    // assume 200 righe
    SetScrollbar(wxVERTICAL, 0, 0, 200, FALSE);
    SetScrollbar(wxHORIZONTAL, 0, 0, 200, FALSE);
//    SetScrollbars(m_widthChar,m_heightChar + 20,64+8,200);
    printBuf = new PrintBuffer();
}

DumpWindow::~DumpWindow()
{
	delete printBuf;
//    free(m_text);
}

void DumpWindow::OnSize( wxSizeEvent &event )
{
    m_xChars = (event.GetSize().x - 2*m_xMargin) / m_widthChar;
    m_yChars = (event.GetSize().y - 2*m_yMargin) / m_heightChar;
    if ( !m_xChars )
        m_xChars = 1;
    if ( !m_yChars )
        m_yChars = 1;

//    free(m_text);
//    m_text = (wxChar *)calloc(m_xChars * m_yChars, sizeof(wxChar));
    unsigned long pos, sz = 10;
    if (stream)
    	stream->GetScroll(pos,sz);
    SetScrollbar(wxVERTICAL, GetScrollPos(wxVERTICAL), m_yChars, (sz+31)/16);
    SetScrollbar(wxHORIZONTAL, GetScrollPos(wxHORIZONTAL), m_xChars, 16*4+12);
    y_scroll = GetScrollPos(wxVERTICAL) * m_heightChar;
    x_scroll = GetScrollPos(wxHORIZONTAL) * m_widthChar;

    event.Skip();
}

static const char hexstring[] = "0123456789abcdef";

static inline char* outhex(char* p,unsigned char b)
{
	p[0] = hexstring[b >> 4];
	p[1] = hexstring[b & 15];
	return p+2;
}

// NB: this method is horrible inefficient especially because the caret
//     needs to be redrawn often and in this case we only have to redraw
//     the caret location and not the entire window - in a real program we
//     would use GetUpdateRegion() and iterate over rectangles it contains
void DumpWindow::OnPaint( wxPaintEvent &WXUNUSED(event) )
{
    wxPaintDC dc( this );
    PrepareDC( dc );
    dc.Clear();

    if (!stream) return;
    // !!!
    char cbuf[32];

    // calc begin line
    int x,y;
//    GetViewStart(&x,&y);
    x = GetScrollPos(wxHORIZONTAL);
    y = GetScrollPos(wxVERTICAL);
    stream->SetScrollPos(y*16);

    int xx,yy;
    // FIXME this assume row are all same size !!!
//    CalcUnscrolledPosition(0,0,&xx,&yy);
    xx = m_widthChar * x;
    yy = (m_heightChar/*+20*/) * y;

    wxLogDebug("y=%d yy=%d\n", y, yy);

    dc.SetDeviceOrigin(-xx,-yy);
    dc.SetFont( m_font );
//    dc.SetBackgroundMode( wxSOLID );
//    dc.SetTextBackground( wxColour(0,128,255) );

    wxSize sz(GetClientSize());
    wxRect view(wxPoint(xx,yy), sz);
    // start printing at 0, yy with view area (xx,yy)-(xx+sz.GetWidth(),yy+sz.GetHeight())
    PrintBuffer::Painter painter(*printBuf, dc, y, wxPoint(0,yy), view);
//    prn.SetDefaultLineSpace(20);

    yy += sz.GetHeight();

    wxColour addrColor(0,0,0);
    wxColour hexColor(0,0,255);
    wxColour printColor(255,0,0);

    while(painter.GetY() <= yy)
    {
    	stream->GetPos(cbuf);
	unsigned char buf[16];
	long readed;
	readed = stream->Read(buf,16);
	if (readed <= 0)
		break;
    	dc.SetTextForeground( addrColor );
    	painter.Print(cbuf);
	painter.Print("\t");
	char printable[17],hexdump[56];
	
    	dc.SetTextForeground( hexColor );
	char *p = hexdump; int i;
	for(i=0; i<readed; ++i)
	{
		p = outhex(p,buf[i]);
		if (isprint(buf[i]))
			printable[i] = (char)buf[i];
		else
			printable[i] = '.';
		*p++ = (i==7)?'-':'\t';
	}
	for(; i<16; ++i)
	{
		strcpy(p,"  \t");
		p += 3;
	}
	*p = 0;
	painter.Print(hexdump);
	printable[readed] = 0;
	
    	dc.SetTextForeground( printColor );
	painter.Print(printable);
	painter.Print("\n");
    }
}

void DumpWindow::OnChar( wxKeyEvent &event )
{
/*
    switch ( event.KeyCode() )
    {
        case WXK_LEFT:
            PrevChar();
            break;

        case WXK_RIGHT:
            NextChar();
            break;

        case WXK_UP:
            PrevLine();
            break;

        case WXK_DOWN:
            NextLine();
            break;

        case WXK_HOME:
            Home();
            break;

        case WXK_END:
            End();
            break;

        case WXK_RETURN:
            Home();
            NextLine();
            break;

        default:
            if ( !event.AltDown() && wxIsprint(event.KeyCode()) )
            {
		CharAt(m_xCaret, m_yCaret) = (wxChar)event.KeyCode();
                NextChar();
            }
            else
            {
                event.Skip();

                // don't refresh
                return;
            }
    }

    wxLogDebug(_T("Caret is at (%d, %d)"), m_xCaret, m_yCaret);

    GetCaret()->Move(m_xMargin + m_xCaret * m_widthChar,
                     m_yMargin + m_yCaret * m_heightChar);

    Refresh();
*/
}

void DumpWindow::SetStream(DumpStream* stream)
{
    this->stream = stream;
    if (stream)
    {
	    unsigned long pos,sz;
	    stream->GetScroll(pos,sz);
	    SetScrollbar(wxVERTICAL, 0, 0, sz, TRUE);
            SetScrollbar(wxHORIZONTAL, 0, 0, 4, TRUE);
    }
//    Scroll(0,0);
    Refresh();
}

void DumpWindow::OnScrollWin( wxScrollWinEvent &event )
{
	wxLogDebug("??\n");

	// scroll by size of first row
	if (event.GetOrientation() == wxVERTICAL) {
		int dy = y_scroll - event.GetPosition() * m_heightChar;
		ScrollWindow(0,dy);
		y_scroll = event.GetPosition() * m_heightChar;
	} else {
		int dx = x_scroll - event.GetPosition() * m_widthChar;
		ScrollWindow(dx,0);
		x_scroll = event.GetPosition() * m_widthChar;
	}
	wxLogDebug("scroll(%d,%d)\n", x_scroll, y_scroll);
}
