// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/image.h"
#include "wx/listctrl.h"
#include "wx/sizer.h"
#include "wx/log.h"
#include "wx/splitter.h"

#include "dumpwin.h"
#include "disasmwin.h"
#include "option.h"

// derived classes

class MyFrame;
class MyApp;

// MyFrame

class MyFrame: public wxFrame
{
public:
    MyFrame();

    void OnAbout( wxCommandEvent &event );
    void OnQuit( wxCommandEvent &event );
    void OnOpenFile( wxCommandEvent &event );
    void OnDisasmOptions( wxCommandEvent &event );
    void OnUnimpl( wxCommandEvent &event );

    DisasmWindow     *m_disasm;
    DumpWindow       *m_dump;
    wxSplitterWindow *splitter;

    DECLARE_DYNAMIC_CLASS(MyFrame)
    DECLARE_EVENT_TABLE()
};

// MyApp

class MyApp: public wxApp
{
public:
    virtual bool OnInit();
};

// main program

IMPLEMENT_APP(MyApp)

// ids

enum
{
	SPLITTER_WINDOW = 100,
	SPLITTER_FRAME,
	CANVAS1,
	CANVAS2
};

// MyFrame

enum
{
	ID_OPENFILE = 108,
	ID_OPENPROJECT,
	ID_SAVEPROJECT,
	ID_QUIT,

	ID_SEARCH,
	ID_SEARCH_AGAIN,
	ID_COPY,

	ID_VIEW_DATA,

	ID_GOTO_ENTRYPOINT,
	ID_GOTO_STARTCODE,
	ID_GOTO_ADDRESS,

	ID_REF_IMPORTS,
	ID_REF_EXPORTS,
	ID_REF_STRINGS,
	
	ID_OPTION_INSTRUCTIONS,
	
	ID_ABOUT
};

IMPLEMENT_DYNAMIC_CLASS( MyFrame, wxFrame )

BEGIN_EVENT_TABLE(MyFrame,wxFrame)
  
  EVT_MENU    (ID_OPENFILE,  MyFrame::OnOpenFile)
  EVT_MENU    (ID_OPENPROJECT,  MyFrame::OnUnimpl)
  EVT_MENU    (ID_SAVEPROJECT,  MyFrame::OnUnimpl)
  EVT_MENU    (ID_QUIT,  MyFrame::OnQuit)

  EVT_MENU    (ID_SEARCH,  MyFrame::OnUnimpl)
  EVT_MENU    (ID_SEARCH_AGAIN,  MyFrame::OnUnimpl)
  EVT_MENU    (ID_COPY,  MyFrame::OnUnimpl)

  EVT_MENU    (ID_VIEW_DATA, MyFrame::OnUnimpl)

  EVT_MENU    (ID_GOTO_ENTRYPOINT, MyFrame::OnUnimpl)
  EVT_MENU    (ID_GOTO_STARTCODE, MyFrame::OnUnimpl)
  EVT_MENU    (ID_GOTO_ADDRESS, MyFrame::OnUnimpl)

  EVT_MENU    (ID_REF_IMPORTS, MyFrame::OnUnimpl)
  EVT_MENU    (ID_REF_EXPORTS, MyFrame::OnUnimpl)
  EVT_MENU    (ID_REF_STRINGS, MyFrame::OnUnimpl)

  EVT_MENU    (ID_OPTION_INSTRUCTIONS, MyFrame::OnDisasmOptions)
	
  EVT_MENU    (ID_ABOUT, MyFrame::OnAbout)
END_EVENT_TABLE()

MyFrame::MyFrame()
       : wxFrame( (wxFrame *)NULL, -1, "PEReaDeR GUI",
                  wxPoint(20,20), wxSize(470,500) )
{
    wxMenuBar *menu_bar = new wxMenuBar( wxMB_DOCKABLE );
    
    wxMenu *menu = new wxMenu();
    menu->Append( ID_OPENFILE, "Open file");
    menu->AppendSeparator();
    menu->Append( ID_OPENPROJECT, "Open project");
    menu->Append( ID_SAVEPROJECT, "Save project");
    menu->AppendSeparator();
    menu->Append( ID_QUIT, "E&xit\tAlt-X");
    menu_bar->Append(menu, "&File");

    menu = new wxMenu();
    menu->Append( ID_COPY, "&Copy");
    menu->AppendSeparator();
    menu->Append( ID_SEARCH, "&Find..");
    menu->Append( ID_SEARCH_AGAIN, "Find &Next");
    menu_bar->Append(menu, "&Edit");
    
    menu = new wxMenu();
    menu->Append(ID_VIEW_DATA, "Dump data", wxEmptyString, TRUE);
    menu_bar->Append(menu, "&View");

    menu = new wxMenu();
    menu->Append(ID_GOTO_ENTRYPOINT, "Entry point");
    menu->Append(ID_GOTO_STARTCODE, "Start code");
    menu->Append(ID_GOTO_ADDRESS, "Address");
    menu_bar->Append(menu, "&Goto");

    menu = new wxMenu();
    menu->Append(ID_REF_IMPORTS, "Imports");
    menu->Append(ID_REF_EXPORTS, "Exports");
    menu->Append(ID_REF_STRINGS, "Strings");
    menu_bar->Append(menu, "&References");

    menu = new wxMenu();
    menu->Append( ID_OPTION_INSTRUCTIONS, "Instruction");
    menu_bar->Append(menu, "&Options");

    menu = new wxMenu();
    menu->Append( ID_ABOUT, "&About");
    menu_bar->Append(menu, "&Help");

    SetMenuBar( menu_bar );

    CreateStatusBar(2);

    splitter = new wxSplitterWindow(this, SPLITTER_WINDOW, wxDefaultPosition, wxSize(200,200), wxSP_3D | wxSP_LIVE_UPDATE);

    int widths[] = { -1, 100 };
    SetStatusWidths( 2, widths );

    m_disasm = new DisasmWindow( splitter, -1, wxDefaultPosition, 
		    wxDefaultSize );
    m_disasm->SetScrollbars( 20, 20, 50, 50 );

    m_dump = new DumpWindow(splitter);
    m_dump->SetStream(CreateSimpleDumpStream());
//    m_dump->SetScrollbars( 10, 10, 50, 100 );

    splitter->Initialize(m_disasm);
    splitter->SplitHorizontally(m_disasm,m_dump);
}

void MyFrame::OnQuit( wxCommandEvent &WXUNUSED(event) )
{
  Close( TRUE );
}

void MyFrame::OnAbout( wxCommandEvent &WXUNUSED(event) )
{
  (void)wxMessageBox( "PeReaDeR GUI\n"
                      "Frediano Ziglio (c) 1996-2002",
                      "About PeReaDeR GUI", wxICON_INFORMATION | wxOK );
}

void MyFrame::OnOpenFile( wxCommandEvent &WXUNUSED(event) )
{
  wxFileDialog dialog(this, "Choose file to disassemble",wxEmptyString,wxEmptyString,
		  "Executable file (*.exe)|*.exe|"
		  "Library file (*.dll)|*.dll|"
		  "System files (*.sys)|*.sys",wxOPEN);
  if (dialog.ShowModal() == wxID_OK)
  {
  	(void)wxMessageBox( dialog.GetPath(),
                      "PeReaDeR GUI", wxICON_INFORMATION | wxOK );
  }
}

static DisasmOptionsRead::Results disasmOptions = 
	{ true, true, true, true };

void MyFrame::OnDisasmOptions( wxCommandEvent &WXUNUSED(event) )
{
	DisasmOptionsRead::Read(this,disasmOptions);
}

void MyFrame::OnUnimpl( wxCommandEvent &WXUNUSED(event) )
{
  (void)wxMessageBox( "Unimplemented function",
                      "PeReaDeR GUI", wxICON_INFORMATION | wxOK );
}

//-----------------------------------------------------------------------------
// MyApp
//-----------------------------------------------------------------------------

bool MyApp::OnInit()
{
	wxFrame *frame = new MyFrame();
	frame->Show( TRUE );

	return TRUE;
}

