// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all <standard< wxWindows headers
#ifndef WX_PRECOMP
    #include <wx/wx.h>

    #include <wx/log.h>
#endif

#include <cassert>
#include "printwin.h"

// ============================================================================
// implementation
// ============================================================================

void PrintWin::RowInfo::SetCol(unsigned int col, char ch, unsigned int size, unsigned spaceSize)
{
        assert(cols.size() == chars.size());

        // silly case, append char
        if (col == cols.size()) {
                cols.resize(col+1);
                chars.resize(col+1);
                cols[col] = col ? cols[col-1] + size : size;
                chars[col] = ch;
                return;
        }

        // set a character inside
        if (col < Size()) {
                int diff = cols[col] - (col ? cols[col-1] : 0) - size;
                if (!diff)
                        return;
                for(unsigned int i = col; i < cols.size(); ++i)
                        cols[i] -= diff;
                return;
        }

        // expand
        assert(col >= Size());
        unsigned oldSize = cols.size();
        cols.resize(col + 1); 
        chars.resize(col + 1);
        unsigned int x = oldSize ? cols[oldSize-1] : 0;
        for (unsigned int i = oldSize; i < col; ++i) {
                assert(i >= oldSize && i < col);
                x += spaceSize;
                cols[i] = x;
                cols[i] = ' ';
        }
        cols[col] = x + size;
        chars[col] = ch;
}

// ----------------------------------------------------------------------------
// PrintWin
// ----------------------------------------------------------------------------

void PrintWin::Print(const char* s)
{
	const char* p = s;
	wxString line;
	for(;*p;++p)
	{
		switch(*p)
		{
		// new line
		case '\n':
			PrintPart(line,true);
			line.Truncate(0);
			break;
		// backslash
		case 8:
			if (line.Length() > 0)
				line.Truncate(line.Length()-1);
			break;
		default:
			line += *p;
		}
	}
	if (line.Length() > 0)
		PrintPart(line);
}

void PrintWin::UpdateRowInfo(const wxString& s)
{
	wxCoord w, h, spaceSize;
        wxString ch;

        // TODO cache somewhere
        dc.GetTextExtent(" ",&spaceSize,&h);

        assert(row >= 0 && column >=0);
        if (rowInfos.size() <= row)
                rowInfos.resize(row + 1);

        assert(row >= 0 && row < rowInfos.size());
        RowInfo& ri = rowInfos[row];
        assert(ri.GetStart() >= 0 && ri.GetStart() <= ri.GetEnd());

        // scan all characters
        for(unsigned i = 0; i < s.Length(); ++i) {
                ch = s[i];
                // TODO cache somewhere
                dc.GetTextExtent(ch,&w,&h);

                ri.SetCol(column + i, s[i], w, spaceSize);
        }
        assert(column + s.Length() == ri.Size());
}

void PrintWin::PrintPart(const wxString& s,bool newLine)
{
	dc.DrawText(s,pt.x,pt.y);
	wxCoord w,h;
	dc.GetTextExtent(s,&w,&h);
	
	if (h > maxHInLine)
	   maxHInLine = h;

        UpdateRowInfo(s);
	
	if (newLine)
	{
                if (!maxHInLine) {
                        dc.GetTextExtent("Mp",&w,&h);
                        maxHInLine = h;
                }
		pt.y += maxHInLine + lineDefaultExtraSpace;
		pt.x = 0;
		maxHInLine = 0;

                /* next row */
                ++row;
                column = 0;
	}
	else
	{
		pt.x += w;
                column += s.Length();
	}
}

/*
 * Gestire una stampa con queste caratteristiche:
 * Ad ogni carattere registra la sua larghezza per poi sapere data una
 * posizione x,y la relativa riga,colonna
 *
 * Gestire i TAB in modo che tutti i TAB siano salvati (un po' come una
 * colonna) e le colonne non si sovrappongano. Deve esserci la possibilita
 * di andare ad un prossimo TAB. Se chiaramente si espande un TAB si deve
 * ricalcolare il tutto solo una volta. Oppure decidere che certe colonne
 * sono dei TAB stop e dare la possibilita' di settare la colonna ?
 *
 * La larghezza dello scroll deve essere dinamica (anche qui ricalcolo solo
 * una volta). Se deve gestire piu' font e piu' colori (salvarseli in
 * disegno?).
 *
 * Inoltre in certi casi sarebbe utile poter lasciare degli spazi extra 
 * per i caratteri per poter inserire contorni o altro (fissare spazio 
 * verticale e TAB ?)
 * */
